@@ -39,7 +39,7 @@
 #include "preferences.h"
 #include "scoreboard.h"
 #include "warp.h"
-#include "games-fullscreen-action.h"
+#include "games-gridframe.h"
 #include "games-scores.h"
 
 #include <clutter-gtk/clutter-gtk.h>
@@ -55,6 +55,7 @@ GSettings *settings;
 GSettings *worm_settings[NUMWORMS];
 GtkWidget *window;
 GtkWidget *statusbar;
+GtkWidget *notebook;
 GtkWidget *chat = NULL;
 
 static const GamesScoresCategory scorecats[] = {
@@ -91,29 +92,14 @@ gint restart_id = 0;
 gint current_level;
 
 gboolean is_paused;
+static gboolean new_game_2_cb (GtkWidget * widget, gpointer data);
 
 static gint add_bonus_cb (gpointer data);
 
-static gint end_game_cb (GtkAction * action, gpointer data);
-
-static GtkAction *new_game_action;
-GtkAction *pause_action;
-static GtkAction *end_game_action;
-static GtkAction *preferences_action;
-static GtkAction *scores_action;
-static GtkAction *fullscreen_action;
-
-static void
-hide_cursor (void)
-{
-  clutter_stage_hide_cursor (CLUTTER_STAGE (stage));
-}
-
-static void
-show_cursor (void)
-{
-  clutter_stage_show_cursor (CLUTTER_STAGE (stage));
-}
+static GSimpleAction *new_game_action;
+static GSimpleAction *end_game_action;
+static GSimpleAction *pause_action;
+static GSimpleAction *preferences_action;
 
 gint
 game_running (void)
@@ -138,13 +124,99 @@ delete_cb (GtkWidget * widget, gpointer data)
 }
 
 static void
-quit_cb (GObject * object, gpointer data)
+activate_toggle (GSimpleAction *action,
+                 GVariant      *parameter,
+                 gpointer       user_data)
 {
-  gtk_widget_destroy (window);
+  GVariant *state;
+
+  state = g_action_get_state (G_ACTION (action));
+  g_action_change_state (G_ACTION (action), g_variant_new_boolean (!g_variant_get_boolean (state)));
+  g_variant_unref (state);
 }
 
 static void
-about_cb (GtkAction * action, gpointer data)
+change_pause_state (GSimpleAction *action,
+                         GVariant      *state,
+                         gpointer       user_data)
+{
+  if (!is_paused) {  //If it's not currently paused, pause the game
+    is_paused = TRUE;
+    if (main_id || restart_id || dummy_id) {
+      if (main_id) {
+        g_source_remove (main_id);
+        main_id = 0;
+      }
+      if (keyboard_id) {
+        g_signal_handler_disconnect (G_OBJECT (stage), keyboard_id);
+        keyboard_id = 0;
+      }
+      if (add_bonus_id) {
+        g_source_remove (add_bonus_id);
+        add_bonus_id = 0;
+      }
+    }
+  }
+  else {  //Resume the game
+    is_paused = FALSE;
+    dummy_id = g_timeout_add (500, (GSourceFunc) new_game_2_cb, NULL);
+  }
+
+}
+
+
+static void
+newgame_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+  new_game ();
+}
+
+static void
+endgame_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+  end_game ();
+}
+
+
+static void
+scores_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+  gnibbles_show_scores (window, 0);
+}
+
+static void
+preferences_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+  gnibbles_preferences_cb (window, user_data);
+}
+
+static void
+help_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+  GError *error = NULL;
+
+  gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (window)), "help:gnome-nibbles", gtk_get_current_event_time (), &error);
+  if (error)
+    g_warning ("Failed to show help: %s", error->message);
+  g_clear_error (&error);
+}
+
+
+
+static void
+about_activated (GSimpleAction *action,
+                 GVariant      *parameter,
+                 gpointer       user_data)
 {
   const gchar *authors[] = { "Sean MacIsaac", "Ian Peters", "Andreas Røsdal",
                              "Guillaume Beland", NULL };
@@ -155,24 +227,42 @@ about_cb (GtkAction * action, gpointer data)
        "program-name", _("Nibbles"),
        "version", VERSION,
        "copyright",
-       "Copyright \xc2\xa9 1999-2008 Sean MacIsaac, Ian Peters, Andreas Røsdal\n"
-       "Copyright \xc2\xa9 2009 Guillaume Beland",
-
+       "Copyright © 1999–2008 Sean MacIsaac, Ian Peters, Andreas Røsdal\n"
+       "Copyright © 2009 Guillaume Beland",
        "license-type", GTK_LICENSE_GPL_2_0, 
-       "comments", _("A worm game for GNOME.\n\nNibbles is a part of GNOME Games."),
+       "comments", _("A worm game for GNOME\n\nNibbles is a part of GNOME Games."),
        "authors", authors,
        "documenters", documenters, 
        "translator-credits", _("translator-credits"), 
        "logo-icon-name", "gnome-nibbles", 
-       "website", "http://www.gnome.org/projects/gnome-games/",
-       "website-label", _("GNOME Games web site"),
-       "wrap-license", TRUE, NULL);
+       "website", "https://wiki.gnome.org/Apps/Nibbles",
+       NULL);
 }
 
+static void
+quit_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+  GApplication *app = user_data;
+
+  g_application_quit (app);
+}
+
+static GActionEntry app_entries[] = {
+  { "newgame", newgame_activated, NULL, NULL, NULL },
+  { "endgame", endgame_activated, NULL, NULL, NULL },
+  { "pause", activate_toggle, NULL, "false", change_pause_state},
+  { "preferences", preferences_activated, NULL, NULL, NULL },
+  { "scores", scores_activated, NULL, NULL, NULL },
+  { "help", help_activated, NULL, NULL, NULL },
+  { "about", about_activated, NULL, NULL, NULL },
+  { "quit", quit_activated, NULL, NULL, NULL },
+};
+
 static gboolean
 key_press_cb (ClutterActor *actor, ClutterEvent *event, gpointer data)
 {
-  hide_cursor ();
 
   if (!(event->type == CLUTTER_KEY_PRESS))
     return FALSE;
@@ -226,7 +316,6 @@ configure_event_cb (GtkWidget *widget, GdkEventConfigure *event, gpointer data)
 static gboolean
 new_game_2_cb (GtkWidget * widget, gpointer data)
 {
-  //if (!is_paused) {  //FIXME figure out why this should be here.
     if (!keyboard_id)
       keyboard_id = g_signal_connect (G_OBJECT (stage),
                                       "key-press-event",
@@ -240,7 +329,6 @@ new_game_2_cb (GtkWidget * widget, gpointer data)
                                     properties->gamespeed,
                                     (GSourceFunc) add_bonus_cb, NULL);
     }
-  //}
 
   dummy_id = 0;
 
@@ -251,10 +339,10 @@ gboolean
 new_game (void)
 {
   int i;
-  gtk_action_set_sensitive (pause_action, TRUE);
-  gtk_action_set_sensitive (end_game_action, TRUE);
-  gtk_action_set_sensitive (new_game_action, FALSE);
-  gtk_action_set_sensitive (preferences_action, FALSE);
+  g_simple_action_set_enabled (new_game_action, FALSE);
+  g_simple_action_set_enabled (end_game_action, TRUE);
+  g_simple_action_set_enabled (pause_action, TRUE);
+  g_simple_action_set_enabled (preferences_action, FALSE);
 
   if (game_running ()) {
     main_id = 0;
@@ -296,40 +384,6 @@ new_game (void)
   return TRUE;
 }
 
-static void
-new_game_cb (GtkAction * action, gpointer data)
-{
-  new_game ();
-}
-
-static void
-pause_game_cb (GtkAction * action, gpointer data)
-{
-  if (!is_paused) {  //If it's not currently paused, pause the game
-    is_paused = TRUE;
-    gtk_action_set_label (action, ("_Resume"));
-    if (main_id || restart_id || dummy_id) {
-      if (main_id) {
-        g_source_remove (main_id);
-        main_id = 0;
-      }
-      if (keyboard_id) {
-        g_signal_handler_disconnect (G_OBJECT (stage), keyboard_id);
-        keyboard_id = 0;
-      }
-      if (add_bonus_id) {
-        g_source_remove (add_bonus_id);
-        add_bonus_id = 0;
-      }
-    }
-  }
-  else {  //Resume the game
-    is_paused = FALSE;
-    gtk_action_set_label (action, ("_Pause"));
-    dummy_id = g_timeout_add (500, (GSourceFunc) new_game_2_cb, NULL);
-  }
-}
-
 void
 end_game (void)
 {
@@ -360,21 +414,14 @@ end_game (void)
 
   animate_end_game ();
 
-  gtk_action_set_sensitive (pause_action, FALSE);
-  gtk_action_set_sensitive (end_game_action, FALSE);
-  gtk_action_set_sensitive (new_game_action, TRUE);
-  gtk_action_set_sensitive (preferences_action, TRUE);
+  g_simple_action_set_enabled (new_game_action, TRUE);
+  g_simple_action_set_enabled (end_game_action, FALSE);
+  g_simple_action_set_enabled (pause_action, FALSE);
+  g_simple_action_set_enabled (preferences_action, TRUE);
 
   is_paused = FALSE;
 }
 
-static gboolean
-end_game_cb (GtkAction * action, gpointer data)
-{
-  end_game ();
-  return FALSE;
-}
-
 static gboolean
 add_bonus_cb (gpointer data)
 {
@@ -464,42 +511,14 @@ main_loop (gpointer data)
     if (winner == -1)
       return FALSE;
 
-    str = g_strdup_printf (_("Game over! The game has been won by %s!"),
-                             names[winner]);
-    g_free (str);
-
-    if (keyboard_id) {
-      g_signal_handler_disconnect (G_OBJECT (stage), keyboard_id);
-      keyboard_id = 0;
-    }
-    if (main_id) {
-      g_source_remove (main_id);
-      main_id = 0;
-    }
-    if (add_bonus_id)
-      g_source_remove (add_bonus_id);
-
-    add_bonus_id = 0;
-
-    animate_end_game ();
     gnibbles_log_score (window);
 
     return FALSE;
   }
 
   if (status == GAMEOVER) {
+    end_game ();
 
-    if (keyboard_id) {
-      g_signal_handler_disconnect (G_OBJECT (stage), keyboard_id);
-      keyboard_id = 0;
-    }
-    main_id = 0;
-    if (add_bonus_id)
-      g_source_remove (add_bonus_id);
-
-    add_bonus_id = 0;
-
-    animate_end_game ();
     gnibbles_log_score (window);
 
     return FALSE;
@@ -558,104 +577,88 @@ main_loop (gpointer data)
   return TRUE;
 }
 
-static gboolean
-show_cursor_cb (GtkWidget * widget, GdkEventMotion *event, gpointer data)
-{
-  show_cursor ();
-  return FALSE;
-}
-
 static void
-help_cb (GtkAction * action, gpointer data)
+activate (GtkApplication* app,
+          gpointer        user_data)
 {
-  GError *error = NULL;
+  GtkWidget *window;
+  GtkWidget *label;
+  GdkGeometry size_hints = {
+    800, 600, 0, 0, 800, 600, 0, 0, 0.0, 0.0, GDK_GRAVITY_NORTH_WEST  
+  };
 
-  gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (window)), "help:gnome-nibbles", gtk_get_current_event_time (), &error);
-  if (error)
-    g_warning ("Failed to show help: %s", error->message);
-  g_clear_error (&error);
-}
-
-static const GtkActionEntry action_entry[] = {
-  {"GameMenu", NULL, N_("_Game")},
-  {"ViewMenu", NULL, N_("_View")},
-  {"SettingsMenu", NULL, N_("_Settings")},
-  {"HelpMenu", NULL, N_("_Help")},
-  {"NewGame", NULL, N_("_New Game"), NULL, NULL,
-   G_CALLBACK (new_game_cb)},
-  {"EndGame", NULL, N_("_End Game"), NULL, NULL,
-   G_CALLBACK (end_game_cb)},
-  {"Pause", NULL, N_("_Pause"), NULL, NULL,
-   G_CALLBACK (pause_game_cb)},
-  {"Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_cb)},
-  {"Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL,
-   G_CALLBACK (gnibbles_preferences_cb)},
-  {"Contents", NULL, N_("_Contents"), NULL, NULL, G_CALLBACK (help_cb)},
-  {"About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (about_cb)}
-};
-
-static const char ui_description[] =
-  "<ui>"
-  "  <menubar name='MainMenu'>"
-  "    <menu action='GameMenu'>"
-  "      <menuitem action='NewGame'/>"
-  "      <menuitem action='EndGame'/>"
-  "      <separator/>"
-  "      <menuitem action='Pause'/>"
-  "      <separator/>"
-  "      <menuitem action='Quit'/>"
-  "    </menu>"
-  "    <menu action='ViewMenu'>"
-  "      <menuitem action='Fullscreen'/>"
-  "    </menu>"
-  "    <menu action='SettingsMenu'>"
-  "      <menuitem action='Preferences'/>"
-  "    </menu>"
-  "    <menu action='HelpMenu'>"
-  "      <menuitem action='Contents'/>"
-  "      <menuitem action='About'/>"
-  "    </menu>"
-  "  </menubar>"
-  "</ui>";
-
-static void
-create_menus (GtkUIManager * ui_manager)
-{
-  GtkActionGroup *action_group;
-
-  action_group = gtk_action_group_new ("MenuActions");
-
-  gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
-  gtk_action_group_add_actions (action_group, action_entry,
-                                G_N_ELEMENTS (action_entry), window);
-
-  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
-  gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
-
-  new_game_action = gtk_action_group_get_action (action_group, "NewGame");
-  scores_action = gtk_action_group_get_action (action_group, "Scores");
-  end_game_action = gtk_action_group_get_action (action_group, "EndGame");
-  pause_action = gtk_action_group_get_action (action_group, "Pause");
-
-  preferences_action = gtk_action_group_get_action (action_group,
-                                                    "Preferences");
-  fullscreen_action = GTK_ACTION (games_fullscreen_action_new ("Fullscreen", GTK_WINDOW(window)));
-  gtk_action_group_add_action_with_accel (action_group, fullscreen_action, NULL);
-
-}
-
-static void
-setup_window (void)
-{
   GtkWidget *vbox;
-  GtkWidget *menubar;
-  GtkWidget *packing;
-
-  GtkUIManager *ui_manager;
-  GtkAccelGroup *accel_group;
   ClutterColor stage_color = {0x00,0x00,0x00,0xff};
 
-  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  GtkWidget *packing;
+
+  GtkBuilder *builder;
+
+  window = gtk_application_window_new (app);
+
+  gtk_window_set_geometry_hints (GTK_WINDOW (window),
+                    window,
+                    &size_hints,
+                    GDK_HINT_MIN_SIZE);
+
+  g_action_map_add_action_entries (G_ACTION_MAP (app), app_entries, G_N_ELEMENTS (app_entries), app);
+
+  builder = gtk_builder_new ();
+
+  gtk_builder_add_from_string (builder,
+                               "<interface>"
+                               "  <menu id='app-menu'>"
+                               "    <section>"
+                               "      <item>"
+                               "        <attribute name='label' translatable='yes'>_New Game</attribute>"
+                               "        <attribute name='action'>app.newgame</attribute>"
+                               "      </item>"
+                               "      <item>"
+                               "        <attribute name='label' translatable='yes'>_End Game</attribute>"
+                               "        <attribute name='action'>app.endgame</attribute>"
+                               "      </item>"
+                               "    </section>"
+                               "    <section>"
+                               "      <item>"
+                               "        <attribute name='label' translatable='yes'>_Pause</attribute>"
+                               "        <attribute name='action'>app.pause</attribute>"
+                               "        <attribute name='accel'>Pause</attribute>"
+                               "      </item>"
+                               "      <item>"
+                               "        <attribute name='label' translatable='yes'>_Preferences</attribute>"
+                               "        <attribute name='action'>app.preferences</attribute>"
+                               "      </item>"
+                               "      <item>"
+                               "        <attribute name='label' translatable='yes'>_Scores</attribute>"
+                               "        <attribute name='action'>app.scores</attribute>"
+                               "      </item>"
+                               "    </section>"
+                               "    <section>"
+                               "      <item>"
+                               "        <attribute name='label' translatable='yes'>_Help</attribute>"
+                               "        <attribute name='action'>app.help</attribute>"
+                               "        <attribute name='accel'>F1</attribute>"
+                               "      </item>"
+                               "      <item>"
+                               "        <attribute name='label' translatable='yes'>_About</attribute>"
+                               "        <attribute name='action'>app.about</attribute>"
+                               "      </item>"
+                               "      <item>"
+                               "        <attribute name='label' translatable='yes'>_Quit</attribute>"
+                               "        <attribute name='action'>app.quit</attribute>"
+                               "        <attribute name='accel'>&lt;Primary&gt;q</attribute>"
+                               "      </item>"
+                               "    </section>"
+                               "  </menu>"
+                               "</interface>", -1, NULL); 
+
+  gtk_application_set_app_menu (GTK_APPLICATION (app), G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu")));
+
+  new_game_action       = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (app) , "newgame"));
+  end_game_action       = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (app) , "endgame"));
+  pause_action          = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (app) , "pause"));
+  preferences_action    = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (app) , "preferences"));
+
   clutter_widget = gtk_clutter_embed_new ();
   stage = gtk_clutter_embed_get_stage (GTK_CLUTTER_EMBED (clutter_widget));
 
@@ -664,75 +667,45 @@ setup_window (void)
   clutter_actor_set_size (CLUTTER_ACTOR (stage),
                           properties->tilesize * BOARDWIDTH,
                           properties->tilesize * BOARDHEIGHT);
-  clutter_stage_set_user_resizable (CLUTTER_STAGE (stage), FALSE);
-
-  board = gnibbles_board_new ();
-
-  gtk_window_set_title (GTK_WINDOW (window), _("Nibbles"));
-
-  gtk_window_set_default_size (GTK_WINDOW (window),
-                               DEFAULT_WIDTH, DEFAULT_HEIGHT);
-  //games_conf_add_window (GTK_WINDOW (window), KEY_PREFERENCES_GROUP);
-
-  g_signal_connect (G_OBJECT (window), "destroy",
-                    G_CALLBACK (gtk_main_quit), NULL);
-  g_signal_connect (G_OBJECT (window), "delete_event",
-                    G_CALLBACK (delete_cb), NULL);
-
-  gtk_widget_realize (window);
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
-  ui_manager = gtk_ui_manager_new ();
-  create_menus (ui_manager);
-
-  accel_group = gtk_ui_manager_get_accel_group (ui_manager);
-  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
-
-  menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
-
-  gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
-
-  g_signal_connect (G_OBJECT (clutter_widget), "configure_event",
-                    G_CALLBACK (configure_event_cb), NULL);
-
-  g_signal_connect (G_OBJECT (window), "focus_out_event",
-                    G_CALLBACK (show_cursor_cb), NULL);
-
-  packing = gtk_aspect_frame_new (NULL,
-                        0.5,
-                        0.5,
-                        1.38,   //between 1.398176292 and 1.388888889
-                        FALSE);
-
-
-  gtk_container_add( GTK_CONTAINER(packing), clutter_widget); 
-
+  //Games Grid Frame Packing
+  packing = games_grid_frame_new (BOARDWIDTH, BOARDHEIGHT);
+  gtk_container_add (GTK_CONTAINER (packing), clutter_widget);
   gtk_box_pack_start (GTK_BOX (vbox), packing, FALSE, TRUE, 0);
 
+  //Statusbar
   statusbar = gtk_statusbar_new ();
   gtk_box_pack_start (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);
 
-  gtk_container_add (GTK_CONTAINER (window), vbox);
+  //Actually inits the board/scoreboard
+  board = gnibbles_board_new ();
+  scoreboard = gnibbles_scoreboard_new (statusbar);
 
+  gtk_container_add (GTK_CONTAINER (window), vbox);
   gtk_widget_show_all (window);
 
-  scoreboard = gnibbles_scoreboard_new (statusbar);
+  g_signal_connect (G_OBJECT (clutter_widget), "configure_event",
+                    G_CALLBACK (configure_event_cb), NULL);
 }
 
+
+
 int
 main (int argc, char **argv)
 {
-  GOptionContext *context;
-  gboolean retval;
   int i;
-  GError *error = NULL;
+  GtkApplication *app;
+  int status;
 
   setlocale (LC_ALL, "");
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  games_scores_startup ();
+
   /*
    * Required because the binary doesn't match the desktop file.
    * Has to be before the call to g_option_context_parse.
@@ -741,18 +714,6 @@ main (int argc, char **argv)
 
   g_set_application_name (_("Nibbles"));
 
-  context = g_option_context_new (NULL);
-  g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
-  g_option_context_add_group (context, gtk_get_option_group (TRUE));
-
-  retval = g_option_context_parse (context, &argc, &argv, &error);
-  g_option_context_free (context);
-  if (!retval) {
-    g_print ("%s\n", error->message);
-    g_error_free (error);
-    exit (1);
-  }
-  
   if (gtk_clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS) {
     GtkWidget *dialog = gtk_message_dialog_new (NULL,
                                                 GTK_DIALOG_MODAL,
@@ -783,16 +744,12 @@ main (int argc, char **argv)
                                  GAMES_SCORES_STYLE_PLAIN_DESCENDING);
 
   properties = gnibbles_properties_new ();
-  setup_window ();
   gnibbles_load_pixmap (properties->tilesize);
 
-  gtk_action_set_sensitive (pause_action, FALSE);
-  gtk_action_set_sensitive (end_game_action, FALSE);
-  gtk_action_set_visible (new_game_action, TRUE);
+  app = gtk_application_new (NULL, G_APPLICATION_FLAGS_NONE);
+  g_signal_connect (app, "activate", G_CALLBACK (activate), NULL);
+  status = g_application_run (G_APPLICATION (app), argc, argv);
+  g_object_unref (app);
 
-  gtk_main ();
-
-  gnibbles_properties_destroy (properties);
-
-  return 0;
+  return status;
 }
